=== modified file 'ChangeLog'
--- ChangeLog	2006-06-21 20:14:00 +0000
+++ ChangeLog	2006-07-06 17:08:45 +0000
@@ -1,3 +1,11 @@
+2006-07-06  Szilveszter Farkas <Szilveszter.Farkas@gmail.com>
+
+    * I'm not dead :)
+    * backend/errors.py: added some exceptions related to fileops.move()
+    * backend/fileops.py: implemented move()
+    * backend/info_helper.py: added diff_helper()
+    * backend/info.py: implemented diff()
+
 2006-06-20  Szilveszter Farkas <Szilveszter.Farkas@gmail.com>
 
     * backend/init.py: moved pull() to backend/update.py

=== modified file 'backend/errors.py'
--- backend/errors.py	2006-06-21 20:14:00 +0000
+++ backend/errors.py	2006-07-06 17:08:45 +0000
@@ -76,6 +76,20 @@
         commit.commit()
     """
 
+class MissingArgumentError(OliveError):
+    """ Occurs when not enough parameters are given.
+    
+    May occur in:
+        fileops.move()
+    """
+
+class MultipleMoveError(OliveError):
+    """ Occurs when moving/renaming more than 2 files, but the last argument is not a directory
+    
+    May occur in:
+        fileops.move()
+    """
+
 class NoChangesToCommitError(OliveError):
     """ Occurs if there are no changes to commit
     

=== modified file 'backend/fileops.py'
--- backend/fileops.py	2006-06-11 19:06:06 +0000
+++ backend/fileops.py	2006-07-06 17:08:45 +0000
@@ -20,9 +20,27 @@
 
 import bzrlib.errors as errors
 
-from errors import (DirectoryAlreadyExists, NoFilesSpecified, NoMatchingFiles,
+from errors import (DirectoryAlreadyExists, MissingArgumentError,
+                    MultipleMoveError, NoFilesSpecified, NoMatchingFiles,
                     NotVersionedError)
 
+def add(file_list):
+    """ Add listed files to the branch. 
+    
+    :param file_list - list of files to be added (using full paths)
+    
+    :return: count of ignored files
+    """
+    import bzrlib.add
+    
+    added, ignored = bzrlib.add.smart_add(file_list)
+    
+    match_len = 0
+    for glob, paths in ignored.items():
+        match_len += len(paths)
+    
+    return match_len
+
 def mkdir(directory):
     """ Create new versioned directory.
     
@@ -39,22 +57,25 @@
         wt, dd = WorkingTree.open_containing(directory)
         wt.add([dd])
 
-def add(file_list):
-    """ Add listed files to the branch. 
-    
-    :param file_list - list of files to be added (using full paths)
-    
-    :return: count of ignored files
+def move(names_list):
+    """ Move or rename given files.
+    
+    :param file_list: if two elements, then rename the first to the second, if more elements then move all of them to the directory specified in the last element
     """
-    import bzrlib.add
-    
-    added, ignored = bzrlib.add.smart_add(file_list)
-    
-    match_len = 0
-    for glob, paths in ignored.items():
-        match_len += len(paths)
-    
-    return match_len
+    from bzrlib.builtins import tree_files
+    
+    if len(names_list) < 2:
+        raise MissingArgumentError
+    tree, rel_names = tree_files(names_list)
+        
+    if os.path.isdir(names_list[-1]):
+        # move into existing directory
+        for pair in tree.move(rel_names[:-1], rel_names[-1]):
+            pass
+    else:
+        if len(names_list) != 2:
+            raise MultipleMoveError
+        tree.rename_one(rel_names[0], rel_names[1])
 
 def remove(file_list, new=False):
     """ Make selected files unversioned.

=== modified file 'backend/info.py'
--- backend/info.py	2006-06-19 00:09:38 +0000
+++ backend/info.py	2006-07-06 17:08:45 +0000
@@ -23,70 +23,93 @@
 
 from errors import (NotBranchError)
 
-def nick(branch, nickname=None):
-    """ Get or set nickname.
-    
-    :param branch: path to the branch
-    
-    :param nickname: if specified, the nickname will be set
-    
-    :return: nickname
-    """
-    try:
-        branch = Branch.open_containing(branch)[0]
-    except errors.NotBranchError:
-        raise NotBranchError
-    
-    if nickname is not None:
-        branch.nick = nickname
-
-    return branch.nick    
-
-def revno(branch):
-    """ Get current revision number for specified branch
-    
-    :param branch: path to the branch
-    
-    :return: revision number
-    """
-    try:
-        revno = Branch.open_containing(branch)[0].revno()
-    except errors.NotBranchError:
-        raise NotBranchError
-    else:
-        return revno
-
-def version():
-    """ Get version information from bzr
-    
-    :return: bzrlib version
-    """
-    return bzrlib.__version__
-
-def whoami(branch=None, email=False):
-    """ Get user's data (name and email address)
-    
-    :param branch: if specified, the user's data will be looked up in the branch's config
-    
-    :param email: if True, only the email address will be returned
-    
-    :return: user info (only email address if email is True)
-    """
+def diff(revision=None, file_list=None, diff_options=None, prefix=None):
+    """ Save the diff into a temporary file.
+    
+    :param revision: a list of revisions (one or two elements)
+    
+    :param file_list: list of files you want to diff
+    
+    :param diff_options: external diff options
+    
+    :param prefix: 0 - p0, 1 - p1, or specify prefixes in the form of old/:new/
+    
+    :return: path to the temporary file which contains the diff output
+    """
+    from tempfile import mkstemp
+    
+    from bzrlib.builtins import internal_tree_files
+    from bzrlib.diff import show_diff_trees
     from bzrlib.workingtree import WorkingTree
     
-    if branch is not None:
-        try:
-            b = WorkingTree.open_containing(u'.')[0].branch
-            config = bzrlib.config.BranchConfig(b)
-        except NotBranchError:
-            config = bzrlib.config.GlobalConfig()
-    else:
-        config = bzrlib.config.GlobalConfig()
-        
-    if email:
-        return config.user_email()
-    else:
-        return config.username()
+    from info_helper import diff_helper
+
+    if (prefix is None) or (prefix == '0'):
+        # diff -p0 format
+        old_label = ''
+        new_label = ''
+    elif prefix == '1':
+        old_label = 'old/'
+        new_label = 'new/'
+    else:
+        if not ':' in prefix:
+            raise errors.BzrError("--diff-prefix expects two values separated by a colon")
+        old_label, new_label = prefix.split(":")
+    
+    try:
+        tree1, file_list = internal_tree_files(file_list)
+        tree2 = None
+        b = None
+        b2 = None
+    except errors.FileInWrongBranch:
+        if len(file_list) != 2:
+            raise errors.BzrCommandError("Files are in different branches")
+
+        tree1, file1 = WorkingTree.open_containing(file_list[0])
+        tree2, file2 = WorkingTree.open_containing(file_list[1])
+    
+        if file1 != "" or file2 != "":
+            # FIXME diff those two files. rbc 20051123
+            raise errors.BzrCommandError("Files are in different branches")
+    
+        file_list = None
+    
+    tmpfile = mkstemp(prefix='olive_')
+    tmpfp = open(tmpfile[1], 'w')
+    
+    if revision is not None:
+        if tree2 is not None:
+            raise errors.BzrCommandError("Can't specify -r with two branches")
+    
+        if (len(revision) == 1) or (revision[1].spec is None):
+            ret = diff_helper(tree1, file_list, diff_options,
+                                   revision[0], 
+                                   old_label=old_label, new_label=new_label,
+                                   output=tmpfp)
+        elif len(revision) == 2:
+            ret = diff_helper(tree1, file_list, diff_options,
+                                   revision[0], revision[1],
+                                   old_label=old_label, new_label=new_label,
+                                   output=tmpfp)
+        else:
+            raise errors.BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
+    else:
+        if tree2 is not None:
+            ret = show_diff_trees(tree1, tree2, tmpfp, 
+                                   specific_files=file_list,
+                                   external_diff_options=diff_options,
+                                   old_label=old_label, new_label=new_label)
+        else:
+            ret = diff_helper(tree1, file_list, diff_options,
+                                   old_label=old_label, new_label=new_label,
+                                   output=tmpfp)
+    
+    tmpfp.close()
+    
+    if ret == 0:
+        return False
+    else:
+        return tmpfile[1]
 
 def info(location):
     """ Get info about branch, working tree, and repository
@@ -192,3 +215,68 @@
         return
     except errors.NoRepositoryPresent:
         pass
+
+def nick(branch, nickname=None):
+    """ Get or set nickname.
+    
+    :param branch: path to the branch
+    
+    :param nickname: if specified, the nickname will be set
+    
+    :return: nickname
+    """
+    try:
+        branch = Branch.open_containing(branch)[0]
+    except errors.NotBranchError:
+        raise NotBranchError
+    
+    if nickname is not None:
+        branch.nick = nickname
+
+    return branch.nick    
+
+def revno(branch):
+    """ Get current revision number for specified branch
+    
+    :param branch: path to the branch
+    
+    :return: revision number
+    """
+    try:
+        revno = Branch.open_containing(branch)[0].revno()
+    except errors.NotBranchError:
+        raise NotBranchError
+    else:
+        return revno
+
+def version():
+    """ Get version information from bzr
+    
+    :return: bzrlib version
+    """
+    return bzrlib.__version__
+
+def whoami(branch=None, email=False):
+    """ Get user's data (name and email address)
+    
+    :param branch: if specified, the user's data will be looked up in the branch's config
+    
+    :param email: if True, only the email address will be returned
+    
+    :return: user info (only email address if email is True)
+    """
+    from bzrlib.workingtree import WorkingTree
+    
+    if branch is not None:
+        try:
+            b = WorkingTree.open_containing(u'.')[0].branch
+            config = bzrlib.config.BranchConfig(b)
+        except NotBranchError:
+            config = bzrlib.config.GlobalConfig()
+    else:
+        config = bzrlib.config.GlobalConfig()
+        
+    if email:
+        return config.user_email()
+    else:
+        return config.username()

=== modified file 'backend/info_helper.py'
--- backend/info_helper.py	2006-06-19 00:09:38 +0000
+++ backend/info_helper.py	2006-07-06 17:08:45 +0000
@@ -244,3 +244,42 @@
         ret['size'] = t
 
     return ret
+
+def diff_helper(tree, specific_files, external_diff_options, 
+                    old_revision_spec=None, new_revision_spec=None,
+                    old_label='a/', new_label='b/', output=None):
+    """ Helper for diff.
+
+    :param tree: a WorkingTree
+
+    :param specific_files: the specific files to compare, or None
+
+    :param external_diff_options: if non-None, run an external diff, and pass it these options
+
+    :param old_revision_spec: if None, use basis tree as old revision, otherwise use the tree for the specified revision. 
+
+    :param new_revision_spec:  if None, use working tree as new revision, otherwise use the tree for the specified revision.
+    """
+    import sys
+    
+    from bzrlib.diff import show_diff_trees
+    
+    if output == None:
+        output = sys.stdout
+    
+    def spec_tree(spec):
+        revision_id = spec.in_store(tree.branch).rev_id
+        return tree.branch.repository.revision_tree(revision_id)
+    if old_revision_spec is None:
+        old_tree = tree.basis_tree()
+    else:
+        old_tree = spec_tree(old_revision_spec)
+
+    if new_revision_spec is None:
+        new_tree = tree
+    else:
+        new_tree = spec_tree(new_revision_spec)
+
+    return show_diff_trees(old_tree, new_tree, output, specific_files,
+                           external_diff_options,
+                           old_label=old_label, new_label=new_label)

=== modified file 'backend/update.py'
--- backend/update.py	2006-06-21 20:14:00 +0000
+++ backend/update.py	2006-07-06 17:08:45 +0000
@@ -172,4 +172,3 @@
             return 0
     finally:
         tree.unlock()
-

